package menu;

import javax.microedition.lcdui.*;
import java.util.*;

/**
* A MenuPage is used to hold lists of other Pages.  The
* user selects a Page from the list and then the new Page
* is shown on the display.
* <p>MenuPage can be used as the base class for dynamic Page
* classes.  It uses synchronized blocks to control access to
* the run variable and other variables that are shared between
* threads, and it sets the run variable to false if the user
* requests that we go to another page.
*
* <p>$Id: MenuPage.java,v 1.3 2003/01/21 00:22:07 finson Exp $
*/
public class MenuPage extends Page {
	/**
	* Initialize a new MenuPage object using the supplied parameters.
	* @param label short label for this page (as might be used in
	* a menu)
	* @param title longer label for this page (as might be used in
	* a screen title)
	* @param icon small icon for this page (as might be used in
	* a toolbar or menu
	* @param hasExit true if this menu page should have an Exit command.
	*/
	public MenuPage(String label,String title,Image icon,boolean hasExit) {
		super(label,title,icon);
		this.child = new Vector();
		this.hasExit = hasExit;
	}
	/**
	* Initialize a new MenuPage object using the supplied parameters
	* and default values where needed.
	* The title defaults to null (meaning that the label will be used
	* in its place).  
	* The icon defaults to null (meaning text only will be displayed).
	* @param label short label for this page (as might be used in
	* a menu)
	* @param hasExit true if this menu page should have an Exit command.
	*/
	public MenuPage(String label,boolean hasExit) {
		this(label,null,null,hasExit);
	}
	/**
	* Initialize a new MenuPage object using the supplied parameters
	* and default values where needed.  
	* The title defaults to null (meaning that the label will be used
	* in its place).  
	* The icon defaults to null (meaning text only will be displayed).
	* hasExit defaults to false, meaning that there is no Exit command
	* @param label short label for this page (as might be used in
	* a menu)
	*/
	public MenuPage(String label) {
		this(label,null,null,false);
	}
	/**
	* Add a Page to the list of Pages that are children of this menu.
	* The label of each child page is shown when this MenuPage is
	* displayed, and then one of them will be selected by the user.
	* @param p the Page to add to the list of child Pages
	*/
	public synchronized void addChild(Page p) {
		child.addElement(p);
		template.append(p.getLabel(),p.getIcon());
	}
	/**
	* Remove all children from the list for this MenuPage.
	*/
	public synchronized void removeAllChildren() {
		int n = template.size();
		for (int i=n-1; i >= 0; i--) {
			child.removeElementAt(i);
			template.delete(i);
		}
	}
	/**
	* Process the user command and return an integer value telling
	* the PageMgr what to do next.  This method handles the details
	* of a command for a page, while the commandAction method in the
	* PageMgr handles the general responsibility of switching displays.
	* @param c a Command object identifying the command.
	* @param d the Displayable on which this event has occurred
	* @return a pageAction code, selected from the public fields defined
	* by PageMgr.
	*/
	public int doCommandAction(Command c,Displayable d) {
		int pageAction;
		if (d != template) {
			throw new IllegalArgumentException(
			"Unknown Displayable received in "+
			this.getClass().getName());
		}
		synchronized (this) {
			if (c == List.SELECT_COMMAND) {
				int idx = template.getSelectedIndex();
				if (idx >= 0) {
					linkedPage = (Page)child.elementAt(idx);
					run = false;					// request content thread exit
					pageAction = PageMgr.LINK;
				} else {
					pageAction = PageMgr.NONE;
				}
			} else if (c == PageMgr.BACK_COMMAND) {
				run = false;						// request content thread exit
				pageAction = PageMgr.BACK;
			} else if (c == PageMgr.EXIT_COMMAND) {
				run = false;						// request content thread exit
				pageAction = PageMgr.EXIT;
			} else if (c == PageMgr.CANCEL_COMMAND) {
				run = false;						// request content thread exit
				pageAction = PageMgr.NONE;
			} else {
				throw new IllegalArgumentException(
				"Unknown Command received in "+
				this.getClass().getName());
			}
		}
		return pageAction;
	}
	/**
	* Return the Page that this Page feels should be the next page
	* to be displayed.  Note that the return value was set during
	* the doCommandAction execution that immediately preceded this
	* call.
	* @return Page the next Page to be displayed
	*/
	public synchronized Page getLinkedPage() {
		return linkedPage;
	}
	/**
	* Return the Displayable object for this MenuPage, partially
	* filled in and ready to display.
	* @return the appropriate Displayable object for this Page
	*/
	public synchronized Displayable getDisplayable() {
		String t = (getTitle()==null) ? getLabel() : getTitle();
		template.setTitle(t);
		if (hasExit) {
			template.addCommand(PageMgr.EXIT_COMMAND);
		} else {
			template.removeCommand(PageMgr.EXIT_COMMAND);
		}
		if (getPrevious() != null) {
			template.addCommand(PageMgr.BACK_COMMAND);
		} else {
			template.removeCommand(PageMgr.BACK_COMMAND);
		}
		return template;
	}

	//----------------------------------------------------------------------
	
	/** the template that is filled out to be our Displayable */
	protected MenuPageScreen template = new MenuPageScreen();

	/** list of child Pages for this menu page */
	private Vector child;
	/** the next Page that the user would like to see */
	private Page linkedPage;
	/** true if the screen should have an Exit command */
	private boolean hasExit;

	//--------------------------------------------------------------------
	
	/**
	* A MenuPageScreen is the Displayable (the view) for a
	* MenuPage (the model).
	*/
	protected class MenuPageScreen extends List {
		MenuPageScreen() {
			super("Menu Template",List.IMPLICIT);
		}
	}
}
