package store;

import javax.microedition.rms.*;
import java.io.*;

/**
* A StorableObject contains a reference to an object that implements
* the Storable interface, and also contains the name of the record 
* store and specific record id where this object is kept.
* <p>$Id: StorableObject.java,v 1.1 2003/01/19 18:06:27 finson Exp $
*/
public class StorableObject {
	/**
	* Create a new StorableObject.
	* @param s the String name of the RecordStore where the user data
	* is kept
	* @param id the record id for this user data
	* @param data the actual user data object kept in the record store
	*/
	public StorableObject(String s, int id, Storable data) {
		storeName = s;
		recordID = id;
		userdata = data;
	}
	/**
	* Get the record store name associated with the user data object.
	* @return the name of the record store where this item is kept
	*/
	public String getStoreName() {
		return storeName;
	}
	/**
	* Get the record id associated with the user data object.
	* @return the record id for this object. 0 if not written
	* into the record store anywhere yet.
	*/
	public int getRecordID() {
		return recordID;
	}
	/**
	* Get the user data object.
	* @return the user data object
	*/
	public Storable getUserData() {
		return userdata;
	}
	/**
	* Read an array of bytes from the record store, then
	* use the methods provided by a Storable object to update the
	* fields of the object.  Note that it is the responsibility of
	* the caller to only read the correct type of object from the 
	* RecordStore.  If the record has no data, this method returns
	* without changing the object (and without throwing an exception).
	* @throws RecordStoreException if anything bad happens while 
	* getting the record
	* @throws IOException if anything bad happens while reading
	* the record data into the object
	*/
	public void readUserDataObject() throws RecordStoreException, IOException {
		RecordStore rs = RecordStore.openRecordStore(storeName,false);
		byte[] rec = rs.getRecord(recordID);
		rs.closeRecordStore();
		if (rec == null) return;
		DataInputStream is = new DataInputStream(new ByteArrayInputStream(rec));
		userdata.readFields(is);
		is.close();
	}
	/**
	* Write the fields of a Storable object into an array of bytes,
	* then write that array of bytes into the RecordStore.
	* @throws RecordStoreException if anything bad happens while 
	* writing the record
	* @throws IOException if anything bad happens while writing
	* the record data into the byte array
	*/
	public void writeUserDataObject() throws RecordStoreException, IOException {
		RecordStore rs;
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DataOutputStream os = new DataOutputStream(baos);
		userdata.writeFields(os);
		byte[] rec = baos.toByteArray();
		os.close();
		if (recordID == 0) {
			rs = RecordStore.openRecordStore(storeName,true);
			recordID = rs.addRecord(rec,0,rec.length);
		} else {
			rs = RecordStore.openRecordStore(storeName,false);
			rs.setRecord(recordID,rec,0,rec.length);
		}
		rs.closeRecordStore();
	}

	//--------------------------------------------------------------------
	
	/** the name of the RecordStore the user data object is in */
	private String storeName;
	/** the recordID for the user data object */
	private int recordID;
	/** the user data object itself */
	private Storable userdata;
	
}
		

